@@ -1,4 +1,4 @@
-'''
+"""
 Python module to read, play, and write sound data.
 For flexibility, FFMPEG is used for non-WAV files.
 
@@ -6,10 +6,10 @@ If you re-set the location of FFMPEG, please run the following code:
 
 >>> ffmpeg = FFMPEG_info()
 >>> ffmpeg.set()
-    
+
 You can obtain it for free from
     http://ffmpeg.org
-    
+
 Mac users using Anaconda should follow the instructions on
 
         https://anaconda.org/soft-matter/ffmpeg
@@ -20,10 +20,10 @@ Mac users using Anaconda should follow the instructions on
 
     seemed to work. Binaries are also available from
 
-        - http://www.evermeet.cx/ffmpeg/ffmpeg-3.2.4.7z
-        - http://www.evermeet.cx/ffplay/ffplay-3.2.4.7z
+        - http://www.evermeet.cx/ffmpeg/ffmpeg-4.1.1.7z
+        - http://www.evermeet.cx/ffplay/ffplay-4.1.1.7z
 
-Note that FFMPEG must be installed externally!
+Note that FFMPEG must be installed externally, not as a Python package!!
 Please install ffmpeg/ffplay in the following directory:
 
     - Windows:  "C:\\\\Program Files\\\\ffmpeg\\\\bin\\\\"
@@ -32,10 +32,10 @@ Please install ffmpeg/ffplay in the following directory:
 
 Compatible with Python >=3.5
 
-'''
+"""
 
 '''
-Date:   March-2018
+Date:   March-2021
 Author: thomas haslwanter
 
 '''
@@ -64,6 +64,8 @@ if file_dir not in sys.path:
     
 import misc
 
+from sksound import misc
+
 # On Win playing sound works automatically
 # For the other packages you need the module "pygame"
 
@@ -71,25 +73,28 @@ if sys.platform=='win32':
     import winsound
 elif sys.platform == 'linux':
     import pygame
-    
+
+
 class NoFFMPEG_Error(Exception):
-    pass    
-    
+    pass
+
+
 class FFMPEG_info:
-    '''
+    """
+
     Class for storing the config-info for FFMPEG.
-    
+
     If first checks if FFMPEG is installed. FFMPEG is necessary to
     read MP3-files etc. Once checked, the corresponding information
     is saved in "ffmpeg.json", under "/FFMPEG_info/sksound/":
-    
+
     FFMPEG_info properties:
         - config_file : JSON-file, with the config-information
         - ffmpeg : Commandline location of the command "ffmpeg"
         - ffplay : Commandline location of the command "ffplay"
-    
-    
-    '''
+
+
+    """
 
     def __init__(self):
         """Set the name of the config-file, and the properties
@@ -120,6 +125,7 @@ class FFMPEG_info:
                 self.ffmpeg = info['ffmpeg']
                 self.ffplay = info['ffplay']
         
+                
     def set(self):
         """
         Set the config-filename, and write the FFMPEG_info
@@ -166,8 +172,10 @@ class FFMPEG_info:
         
         return
     
+    
 class Sound:
-    '''
+    """
+
     Class for working with sound in Python.
     
     A Sound object can be initialized
@@ -205,11 +213,19 @@ class Sound:
         - duration
         - bitsPerSample
 
+    SoundMethods:
+        - generate_sound
+        - get_info
+        - play
+        - read_sound
+        - summary
+        - write_wav
+        
     Examples
     --------
     >>> from sksound.sounds import Sound
-    >>> mySound1 = Sound()
-    >>> mySound2 = Sound('test.wav')
+    >>> mySound1 = Sound()                  # here the user is prompted for an input file
+    >>> mySound2 = Sound('test.wav')     # here the input file is provided directly
     >>>
     >>> rate = 22050
     >>> dt = 1./rate
@@ -220,14 +236,13 @@ class Sound:
     >>> sounddata = np.int16(x*amp)
     >>> mySound3 = Sound(inData=sounddata, inRate=rate)
 
-    '''
-    
+    """
 
     def __init__(self, inFile = None, inData = None, inRate = None):
-        '''Initialize a Sound object '''
+        """ Initialize a Sound object """
         
         # Information about FFMPEG
-        self.info = FFMPEG_info()
+        self.ffmpeg_info = FFMPEG_info()
         
         if inData is not None:
             if inRate is None:
@@ -248,9 +263,10 @@ class Sound:
                 self.source = inFile
                 self.read_sound(self.source)
         
-
+                
     def read_sound(self, inFile):
-        '''
+        """
+
         Read data from a sound-file.
 
         Parameters
@@ -275,8 +291,8 @@ class Sound:
         >>> mySound.play()
         >>> mySound.read_sound('test2.wav') # If you want to read in another(!) file
 
-        '''
-        
+        """
+
         # Python can natively only read "wav" files. To be flexible, use "ffmpeg" for conversion for other formats
         if not os.path.exists(inFile):
             print('{0} does not exist!'.format(inFile))
@@ -284,14 +300,14 @@ class Sound:
        
         (root, ext) = os.path.splitext(inFile)
         if ext[1:].lower() != 'wav':
-            if self.info.ffmpeg == None:
+            if self.ffmpeg_info.ffmpeg == None:
                 print('Sorry, need FFMPEG for non-WAV files!')
                 self.rate = None
                 self.data = None
                 raise NoFFMPEG_Error
                 
             outFile = root + '.wav'
-            cmd = [self.info.ffmpeg, '-i', inFile, outFile, '-y']
+            cmd = [self.ffmpeg_info.ffmpeg, '-i', inFile, outFile, '-y']
             subprocess.run(cmd)
             print('Infile converted from ' + ext + ' to ".wav"')
             
@@ -312,33 +328,32 @@ class Sound:
             
         self._setInfo()
         print('data read in!')
-    
+
 
     def play(self):
-        '''
-        Play the stored sound
+        """
+       Play the stored sound
 
-        Parameters
-        ----------
-        None : 
+       Parameters
+       ----------
+       None :
 
-        Returns
-        -------
-        None :
-            
+       Returns
+       -------
+       None :
 
-        Notes
-        -----
-        On "Windows" the module "winsound" is used; on "Linux" I use
-        "pygame"; and on "OSX" the terminal command "afplay".
 
-        Examples
-        --------
-        >>> mySound = Sound()
-        >>> mySound.read_sound('test.wav')
-        >>> mySound.play()
+       Notes
+       -----
+       On "Windows" the module "winsound" is used; on "Linux" I use
+       "pygame"; and on "OSX" the terminal command "afplay".
 
-        '''
+       Examples
+       --------
+       >>> mySound = Sound('test.wav')
+       >>> mySound.play()
+
+        """
 
         try:
             if self.source is None:
@@ -360,7 +375,7 @@ class Sound:
                     time.sleep(self.duration)
                     
                     # If you want to use FFMPEG instead, use the following commands:
-                    #cmd = [self.info.ffplay, '-autoexit', '-nodisp', '-i', tmpFile.name]
+                    #cmd = [self.ffmpeg_info.ffplay, '-autoexit', '-nodisp', '-i', tmpFile.name]
                     #subprocess.run(cmd)
                     
             elif os.path.exists(self.source):
@@ -368,7 +383,7 @@ class Sound:
                 print('Playing ' + self.source)
                 
                 # ... then play that one
-                if sys.platform=='win32':
+                if sys.platform == 'win32':
                     winsound.PlaySound(str(self.source), winsound.SND_FILENAME)
                 elif sys.platform == 'darwin':
                     cmd = ['afplay', str(self.source)]
@@ -380,17 +395,17 @@ class Sound:
                     time.sleep(self.duration)
                     
                     # If you want to use FFMPEG instead, use the following commands:
-                    #cmd = [self.info.ffplay, '-autoexit', '-nodisp', '-i', self.source]
+                    #cmd = [self.ffmpeg_info.ffplay, '-autoexit', '-nodisp', '-i', self.source]
                     #subprocess.run(cmd)
                     
         except SystemError:
             print('If you don''t have FFMPEG available, you can e.g. use installed audio-files. E.g.:')
             print('import subprocess')
             print('subprocess.run([r"C:\Program Files (x86)\VideoLAN\VLC\vlc.exe", r"C:\Music\14_Streets_of_Philadelphia.mp3"])')
-            
 
+            
     def generate_sound(self, data, rate):
-        ''' Set the properties of a Sound-object. '''
+        """ Set the properties of a Sound-object. """
 
         # If the data are not in an integer format (if they are e.g. "float"), convert
         # them to integer and scale them to a reasonable amplitude
@@ -404,9 +419,10 @@ class Sound:
         self.source = None
         self._setInfo()
 
+        
+    def write_wav(self, full_out_file = None):
+        """
 
-    def write_wav(self, full_out_file = None):            
-        '''
         Write sound data to a WAV-file.
 
         Parameters
@@ -423,31 +439,31 @@ class Sound:
 
         Examples
         --------
-        >>> mySound = Sound()
-        >>> mySound.read_sound('test.wav')
+        >>> mySound = Sound('test.wav')
         >>> mySound.write_wav()
 
-        '''
+        """
 
         if full_out_file is None:
             
-            (my_file, my_path) = misc.save_file(FilterSpec='*.wav', DialogTitle='Write sound to ...', 
+            (out_file, out_dir) = misc.save_file(FilterSpec='*.wav', DialogTitle='Write sound to ...', 
                           DefaultName='')
-            full_out_file = os.path.join(my_file, my_path)
+            full_out_file = os.path.join(out_dir, out_file)
             if full_out_file is None:
                 print('Output discarded.')
                 return 0
         else:
-            (outFile , outDir) = os.path.split(full_out_file)
+            (out_file , out_dir) = os.path.split(full_out_file)
 
-            write(str(full_out_file), int(self.rate), self.data)
-            print('Sounddata written to ' + outFile + ', with a sample rate of ' + str(self.rate))
-            print('OutDir: ' + outDir)
-
-            return full_out_file
+        write(str(full_out_file), int(self.rate), self.data)
+        print('Sounddata written to ' + out_file + ', with a sample rate of ' + str(self.rate))
+        print('OutDir: ' + out_dir)
+        
+        return full_out_file
+    
     
     def get_info(self):
-        '''
+        """
         Return information about the sound.
 
         Parameters
@@ -465,13 +481,12 @@ class Sound:
             
         Examples
         --------
-        >>> mySound = Sound()
-        >>> mySound.read_sound('test.wav')
+        >>> mySound = Sound('test.wav')
         >>> info = mySound.get_info()
-        >>> (source, rate, numChannels, totalSamples, duration, bitsPerSample) = mySound.info()
+        >>> (source, rate, numChannels, totalSamples, duration, bitsPerSample) = mySound.get_info()
+
+        """
 
-        '''
-        
         return (self.source,
                 self.rate,
                 self.numChannels,
@@ -479,8 +494,9 @@ class Sound:
                 self.duration,
                 self.dataType)
 
+    
     def summary(self):
-        '''
+        """
         Display information about the sound.
 
         Parameters
@@ -497,8 +513,8 @@ class Sound:
         >>> mySound.read_sound('test.wav')
         >>> mySound.summary()
 
-        '''
-        
+        """
+
         import yaml
         
         (source, rate, numChannels, totalSamples, duration, dataType) = self.get_info()
@@ -510,9 +526,10 @@ class Sound:
                 'DataType':dataType}
         print(yaml.dump(info, default_flow_style=False))
         
+        
     def _setInfo(self):
-        '''Set the information properties of that sound'''
-            
+        """ Set the information properties of that sound """
+
         if len(self.data.shape)==1:
             self.numChannels = 1
             self.totalSamples = len(self.data)
@@ -524,76 +541,74 @@ class Sound:
         self.dataType = str(self.data.dtype)
         
     def _selectInput(self):
-        '''GUI for the selection of an in-file. '''
+        """ GUI for the selection of an in-file. """
 
         (my_file, my_path) = misc.get_file(FilterSpec='*.wav', 
                                     DialogTitle='Select sound-input:', 
                                     DefaultName='')
-        full_in_file = os.path.join(my_path, my_file)
-        # filetypes=['*.wav', '*.mp3'])
-        
-        if full_in_file is None:
+        if my_path == 0:
             print('No file selected')
             return 0
         else:
+            full_in_file = os.path.join(my_path, my_file)
             print('Selection: ' + full_in_file)
             return full_in_file
 
+        
 def main():
-    ''' Main function, to test the module '''
+    """ Main function, to test the module """
+
     import os
     import numpy as np
-    import matplotlib.pyplot as plt
-    
+
     # Re-set FFMPEG
-    #ffmpeg = FFMPEG_info()
-    #ffmpeg.set()
-    
+    # ffmpeg = FFMPEG_info()
+    # ffmpeg.set()
+
     # Import a file, and play the sound
-    #dataDir = r'/home/thomas/Coding/scikit-sound/sksound/tests'
-    dataDir = 'tests'
-    inFile = 'a1.wav'
-    
-    fullFile = os.path.join(dataDir, inFile)
-    fullFile2 = os.path.join(dataDir, 'float_sound.wav')
+    # data_dir = r'/home/thomas/Coding/scikit-sound/sksound/tests'
+    data_dir = 'tests'
+    in_file = 'a1.wav'
+
+    full_file = os.path.join(data_dir, in_file)
     try:
-        #mySound = Sound(fullFile)
-        #mySound.play()
-        #mySound.read_sound(fullFile2)
-        #mySound.play()
-        #time.sleep(mySound.duration)
+        # mySound = Sound(full_file)
+        # mySound.play()
+        # time.sleep(mySound.duration)
         mySound2 = Sound()
         mySound2.play()
     except NoFFMPEG_Error:
         pass
-    
-    ## Test with self-generated data
+
+    # Test with self-generated data
     rate = 22050
     dt = 1./rate
     t = np.arange(0,0.5,dt)
     freq = 880
     x = np.sin(2*np.pi*freq*t)
     sounddata = np.int16(x*2**13)
-    
-    inSound = Sound(inData=sounddata, inRate=rate)
-    inSound.summary()
-    inSound.play()
-    time.sleep(inSound.duration)
-    
+
+    in_sound = Sound(inData=sounddata, inRate=rate)
+    in_sound.summary()
+    in_sound.play()
+    time.sleep(in_sound.duration)
+
     print('hi')
-    
-    ## Test if type conversion works
-    inSound2 = Sound(inData=x, inRate=rate)
-    inSound2.play()
-    
+
+    # Test if type conversion works
+    in_sound2 = Sound(inData=x, inRate=rate)
+    in_sound2.play()
+
     # Test with GUI
-    inSound = Sound()
-    inSound.play()
-    print(inSound.summary())
-    out = inSound.get_info()
+    in_sound = Sound()
+    in_sound.play()
+    print(in_sound.summary())
+    out = in_sound.get_info()
     print(out)
-    inSound.write_wav()
-    
+    in_sound.write_wav()
+
+
 if __name__ == '__main__':
-    main()
-    
+    my_sound = Sound()
+    #main()
+
